



pro utplot_goesp_demo, p0, p1, goesp, xray=xray, nodefcolors=nodefcolors, $
        pchannels=pchannels, echannels=echannels, hour_window=hour_window, $
        proton_only=proton_only, electron_only=electron_only, zbuffer=zbuffer, $
        _extra=_extra, reverse=reverse, goes8=goes8, goes9=goes9, $
        nolegend=nolegend, debug=debug

;+ 
;   Name: utplot_goesp_demo
;
;   Purpose: demo read & plot GOES proton/electron values
;
;   Input Parameters:
;      p0 - Tstart for  read/plot OR vector of GOES particle structures (rd_goesp_ascii output)
;      p1 - Tstop time for read/plot GOES particle structure.
;
;   Output Parameters:
;      goesp - optional output GOES particle records (if p0/p1 = time range)
;     
;   Keyword Parameters:
;      nodefcolors - (switch) - if set, dont use "fancy" colors
;      reverse - (switch) - implies /NODEFCOLORS - true for white on black
;      pchannels - (optional) - array of Proton/Particle channels to plot ( def=indgen(6) )
;      echannels - (optional) - array of Electron channesl to plot ( def=indgen(3) )
;      hour_window - optional time window (in hours) to plot (1 or 2 elem array)
;                    (ignored if time range or structure are input)
;      _extra - unrecognized keywords -> utplot via inheritance
;
;   Calling Sequence:
;      IDL> utplot_goesp_demo, time0, time1 [,/xray] [,pchannels=parray] [,echannels=earray]
;
;   Calling Examples:
;      IDL> utplot_goesp_demo, time0, time1, [OPTIONS] ; user supplied time range  
;           -OR- 
;      IDL> utplot_goesp_demo, goesp=goesp   [OPTIONS] ; user supplied GOESP structures    
;                                                      ; ( ie, rd_goesp_ascii output)
;
;   Referenced Routines:
;      rd_goesp_ascii, rd_gxd, time_window, linecolors, wdef, required_tags
;
;   History:
;      17-July-2000 - demo use of 'rd_goesp_ascii' and/or do some summary plots
;-

if not keyword_set(hour_window) then hour_window=12        ; def= +/- 12 hours
case 1 of 
   required_tags(p0,'day,mjd,p,elec'): goesp=p0            ; user input rd_goesp_ascii ouput
   n_params() eq 1: time_window,p0,t0,t1,hour=hour_window  ; 
   n_params() eq 2: time_window,[p0,p1],t0,t1              ; user input time range
   else: begin 
      box_message,['Unexpected input',$
      'IDL> utplot_goesp_demo, time0, time1, [OPTIONS] ; user supplied time range',$
      '     -OR-',$
      'IDL> utplot_goesp_demo, goesp=goesp   [OPTIONS] ; user supplied GOESP structures',$
      '                                                ; ( ie, rd_goesp_ascii output) ']
      return
   endcase
endcase

; - read the GOES particle data base via 'rd_goesp_ascii'
if (1-data_chk(goesp,/struct)) and (exist(0) and exist(t0)) then begin
   rd_goesp_ascii,t0,t1,goesp, plables=plables, elables=elables, /goes8   ; << TODO - time adjust Sat#
endif

if not required_tags(goesp,'time,mjd,p,elec') then begin 
   box_message,['Problem (missing?) GOES particle dbase',$
                'See: http://www.lmsal.com/solarsoft/sswdb_description.html']
   return
endif
   
; ---------- set switch states -------------
proton_only=keyword_set(proton_only)
electron_only=keyword_set(electron_only)
reverse=keyword_set(reverse)
nodefcolors=keyword_set(nodefcolors) or reverse
debug=keyword_set(debug)

npc=data_chk(gt_tagval(goesp,/p),/nx)      ; number proton channels available
nec=data_chk(gt_tagval(goesp,/elec),/nx)   ; number electron channels available

; ------------- define channel subsets of interest --------------
if n_elements(pchannels) eq 0 then pchannels=indgen(npc) else pchannels=pchannels>0<(npc-1)
if n_elements(echannels) eq 0 then echannels=indgen(nec) else echannels=echannels>0<(nec-1)

dtemp=!d.name                              ; dont clobber plot device

if not keyword_set(nodefcolors) then begin 
   goodcolors=[4,5,7,9,12,2]
   pcolors=(goodcolors)(0:npc-1)
   ecolors=(shift(goodcolors,-3))(0:nec-1)
   background=11
   maincolor=!p.color
endif else begin 
   pcolors=replicate(([!p.color,0])(reverse),npc)
   ecolors=replicate(([!p.color,0])(reverse),nec)
   background=([0,!p.color])(reverse)
   maincolor=!p.color-background
endelse

wdef,xx,768,512, zbuffer=zbuffer, title='GOES Particle Events'

; ----------- scale plot to maximum (P/E) --------------------
p0max=max((gt_tagval(goesp,/p))(0,*))
e0max=max((gt_tagval(goesp,/elec))(0,*))
case 1 of 
   proton_only:   begin
      init='p(0,*)'
      units="Protons/cm2-s-sr"
      mtitle="GOES Proton Events"
   endcase
   electron_only: begin
      init='elec(0,*)'
      units="Electrons/cm2-s-sr"
      mtile="GOES Electron Events"
   endcase
   else:          begin
      init=(['elec(0,*)','p(0)'])(p0max ge e0max)
      units='Events/cm2-s-sr'
      mtitle="GOES Proton/Electron Events"
   endcase
endcase
ytitle=units

; ------------- initialize the plot ----------------


initcmd='utplot, goesp, goesp.'+init + ',mtitle=mtitle,ytitle=ytitle,color=maincolor,background=background,/xstyle,/nodata,_extra=_extra'
estat=execute(initcmd)

if electron_only then npc=0
if proton_only   then nec=0

npc=npc<n_elements(pchannels)
nec=nec<n_elements(echannels)

ppsym=1           
epsym=4

for i=0, (npc-1) do $
   outplot,goesp,goesp.p(pchannels(i)), color=pcolors(i), psym=ppsym

for i=0,nec-1 do $
   outplot,goesp,reform(goesp.elec(echannels(i))),color=ecolors(i), psym=epsym

if not keyword_set(nolegend) then begin            ; build legend info
   lcols=-1
   llabs=''
   lpsym=-1
   if npc gt 0 then begin                          ; add Proton info
      lcols=[lcols,pcolors(0:npc-1)]
      llabs=[llabs,plables(0:npc-1)]
      lpsym=[lpsym,replicate(ppsym,npc)]
   endif     
   if nec gt 0 then begin                          ; add Electron info
      lcols=[lcols,ecolors(0:nec-1)]
      llabs=[llabs,elables(0:nec-1)]
      lpsym=[lpsym,replicate(epsym,nec)]   
   endif
   if n_elements(lcols) gt 1 then begin 
      legend,llabs(1:*), psym=lpsym(1:*), colors=lcols(1:*)  ; add legend
   endif
endif

if debug then stop,'debug...'

set_plot,dtemp                                      ; restore users plot device

return
end
